library cod_api;

import 'dart:convert';
import 'dart:ffi';

import 'package:http/http.dart' as http;

class _Helpers {
  _Helpers._();

  static String cleanClientName(String gamertag) {
    return Uri.encodeComponent(gamertag);
  }
}

class _Request {
  _Request._();

  static Future<Map> makeRequest(String uri, String cookies) async {
    try {
      var response = await http.Client().get(
          Uri.https('my.callofduty.com', "/api/papi-client" + uri),
          headers: {
            'cookie': cookies,
            'content-type': 'application/json',
            'user-agent':
                'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36'
          });
      var decodedResponse = jsonDecode(utf8.decode(response.bodyBytes)) as Map;
      var data = decodedResponse['data'] as Map;
      return data;
    } catch (e) {
      final Map<String, String> errorMap = {"error": jsonEncode(e.toString())};
      return errorMap;
    } finally {}
  }
}

class _MW {
  Future<Map> fullData(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/stats/cod/v1/title/mw/platform/$platform/$lookupType/$gamertag/profile/type/mp",
        API.cookies);
  }

  Future<Map> combatHistory(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/$lookupType/$gamertag/matches/mp/start/0/end/0/details",
        API.cookies);
  }

  Future<Map> combatHistoryWithDate(String gamertag, Uint64 startTime,
      Uint64 endTime, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/$lookupType/$gamertag/matches/mp/start/$startTime/end/$endTime/details",
        API.cookies);
  }

  Future<Map> breakdown(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/$lookupType/$gamertag/matches/mp/start/0/end/0",
        API.cookies);
  }

  Future<Map> breakdownWithDate(String gamertag, Uint64 startTime,
      Uint64 endTime, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/$lookupType/$gamertag/matches/mp/start/$startTime/end/$endTime",
        API.cookies);
  }

  Future<Map> seasonLoot(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/loot/title/mw/platform/$platform/$lookupType/$gamertag/status/en",
        API.cookies);
  }

  Future<Map> mapList(String platform) async {
    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/ce/v1/title/mw/platform/$platform/gameType/mp/communityMapData/availability",
        API.cookies);
  }

  Future<Map> matchInfo(String matchId, String platform) async {
    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/fullMatch/mp/$matchId/en",
        API.cookies);
  }
}

class _WZ {
  Future<Map> fullData(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/stats/cod/v1/title/mw/platform/$platform/$lookupType/$gamertag/profile/type/wz",
        API.cookies);
  }

  Future<Map> combatHistory(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/$lookupType/$gamertag/matches/wz/start/0/end/0/details",
        API.cookies);
  }

  Future<Map> combatHistoryWithDate(String gamertag, Uint64 startTime,
      Uint64 endTime, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/$lookupType/$gamertag/matches/wz/start/$startTime/end/$endTime/details",
        API.cookies);
  }

  Future<Map> breakdown(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/$lookupType/$gamertag/matches/wz/start/0/end/0",
        API.cookies);
  }

  Future<Map> breakdownWithDate(String gamertag, Uint64 startTime,
      Uint64 endTime, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/$lookupType/$gamertag/matches/wz/start/$startTime/end/$endTime",
        API.cookies);
  }

  Future<Map> matchInfo(String matchId, String platform) async {
    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/mw/platform/$platform/fullMatch/wz/$matchId/en",
        API.cookies);
  }
}

class _CW {
  Future<Map> fullData(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/stats/cod/v1/title/cw/platform/$platform/$lookupType/$gamertag/profile/type/mp",
        API.cookies);
  }

  Future<Map> combatHistory(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/cw/platform/$platform/$lookupType/$gamertag/matches/mp/start/0/end/0/details",
        API.cookies);
  }

  Future<Map> combatHistoryWithDate(String gamertag, Uint64 startTime,
      Uint64 endTime, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/cw/platform/$platform/$lookupType/$gamertag/matches/mp/start/$startTime/end/$endTime/details",
        API.cookies);
  }

  Future<Map> breakdown(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/cw/platform/$platform/$lookupType/$gamertag/matches/mp/start/0/end/0",
        API.cookies);
  }

  Future<Map> breakdownWithDate(String gamertag, Uint64 startTime,
      Uint64 endTime, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/cw/platform/$platform/$lookupType/$gamertag/matches/mp/start/$startTime/end/$endTime",
        API.cookies);
  }

  Future<Map> seasonLoot(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/loot/title/cw/platform/$platform/$lookupType/$gamertag/status/en",
        API.cookies);
  }

  Future<Map> mapList(String platform) async {
    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/ce/v1/title/cw/platform/$platform/gameType/mp/communityMapData/availability",
        API.cookies);
  }

  Future<Map> matchInfo(String matchId, String platform) async {
    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/cw/platform/$platform/fullMatch/mp/$matchId/en",
        API.cookies);
  }
}

class _VG {
  Future<Map> fullData(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/stats/cod/v1/title/vg/platform/$platform/$lookupType/$gamertag/profile/type/mp",
        API.cookies);
  }

  Future<Map> combatHistory(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/vg/platform/$platform/$lookupType/$gamertag/matches/mp/start/0/end/0/details",
        API.cookies);
  }

  Future<Map> combatHistoryWithDate(String gamertag, Uint64 startTime,
      Uint64 endTime, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/vg/platform/$platform/$lookupType/$gamertag/matches/mp/start/$startTime/end/$endTime/details",
        API.cookies);
  }

  Future<Map> breakdown(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/vg/platform/$platform/$lookupType/$gamertag/matches/mp/start/0/end/0",
        API.cookies);
  }

  Future<Map> breakdownWithDate(String gamertag, Uint64 startTime,
      Uint64 endTime, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/vg/platform/$platform/$lookupType/$gamertag/matches/mp/start/$startTime/end/$endTime",
        API.cookies);
  }

  Future<Map> seasonLoot(String gamertag, String platform) async {
    String lookupType = "gamer";

    if (platform == Platforms.battlenet ||
        platform == Platforms.activision ||
        platform == Platforms.uno) {
      gamertag = _Helpers.cleanClientName(gamertag);
    }

    if (platform == Platforms.uno) lookupType = "id";

    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/loot/title/vg/platform/$platform/$lookupType/$gamertag/status/en",
        API.cookies);
  }

  Future<Map> mapList(String platform) async {
    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/ce/v1/title/vg/platform/$platform/gameType/mp/communityMapData/availability",
        API.cookies);
  }

  Future<Map> matchInfo(String matchId, String platform) async {
    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/crm/cod/v2/title/vg/platform/$platform/fullMatch/mp/$matchId/en",
        API.cookies);
  }
}

class _SHOP {
  Future<Map> purchasableItems(String gameId) async {
    return await _Request.makeRequest(
        "/inventory/v1/title/$gameId/platform/psn/purchasable/public/en",
        API.cookies);
  }

  Future<Map> bundleInformation(String title, String bundleId) async {
    return await _Request.makeRequest(
        "/inventory/v1/title/$title/bundle/$bundleId/en", API.cookies);
  }

  Future<Map> battlePassLoot(Uint32 season, String platform) async {
    if (platform == Platforms.uno || platform == Platforms.activision) {
      platform = Platforms.uno;
    }

    return await _Request.makeRequest(
        "/loot/title/mw/platform/$platform/list/loot_season_$season/en",
        API.cookies);
  }
}

class Platforms {
  Platforms._();
  static String all = 'all';
  static String activision = 'acti';
  static String battlenet = 'battle';
  static String psn = 'psn';
  static String steam = 'steam';
  static String uno = 'uno';
  static String xbox = 'xbl';
}

class API {
  API._();

  static String? ssoToken;
  static String cookies = "";

  static bool login(String? _ssoToken) {
    if (_ssoToken == null) return false;
    API.ssoToken = _ssoToken;

    String fakeXSRF = "68e8b62e-1d9d-4ce1-b93f-cbe5ff31a041";

    API.cookies =
        'new_SiteId=cod; ACT_SSO_LOCALE=en_US;country=US;ACT_SSO_COOKIE=$_ssoToken;XSRF-TOKEN=$fakeXSRF;API_CSRF_TOKEN=$fakeXSRF;ACT_SSO_EVENT="LOGIN_SUCCESS:1644346543228";ACT_SSO_COOKIE_EXPIRY=1645556143194;comid=cod;ssoDevId=63025d09c69f47dfa2b8d5520b5b73e4;tfa_enrollment_seen=true;gtm.custom.bot.flag=human;';

    return true;
  }

  static _MW modernWarfare = _MW();
  static _WZ warZone = _WZ();
  static _CW coldWar = _CW();
  static _VG vanguard = _VG();
  static _SHOP store = _SHOP();
}
